import sys
from typing import List

from nmk.model.builder import NmkTaskBuilder
from nmk.utils import run_with_logs


class FormatBuilder(NmkTaskBuilder):
    def build(self, src_files: List[str], src_folders: List[str], line_length: int):
        # Delegate to black
        run_with_logs([sys.executable, "-m", "black", "-l", str(line_length)] + src_files, self.logger)

        # Delegate to isort
        run_with_logs([sys.executable, "-m", "isort"] + src_folders, self.logger)

        # Touch output file
        self.main_output.touch()



from configparser import ConfigParser
from pathlib import Path

from nmk_base.version import VersionResolver
from pkg_resources import DistributionNotFound, get_distribution

__title__ = "nmk-python"
try:
    __version__ = get_distribution(__title__).version
except DistributionNotFound:  # pragma: no cover
    # For debug
    try:
        with (Path(__file__).parent.parent.parent / "setup.cfg").open("r") as f:
            c = ConfigParser()
            c.read_file(f.readlines())
            __version__ = c.get("metadata", "version")
    except Exception:
        __version__ = "unknown"


class NmkPythonVersionResolver(VersionResolver):
    def get_version(self) -> str:
        return __version__



import shutil
import subprocess
import sys
from typing import Dict

from nmk.model.builder import NmkTaskBuilder
from nmk.model.keys import NmkRootConfig


class PytestBuilder(NmkTaskBuilder):
    def build(self, pytest_args: Dict[str, str]):
        # Clean outputs
        for p in self.outputs:
            if p.is_dir():
                shutil.rmtree(p)
            elif p.is_file():  # pragma: no cover
                p.unlink()

        # Compute extra args
        args = []
        for opt_k, opt_v in pytest_args.items():
            if isinstance(opt_v, bool):
                if opt_v:
                    # Simple option
                    args.append(f"--{opt_k}")
            else:
                # Key + value
                args.append(f"--{opt_k}={opt_v}")

        # Invoke pytest
        subprocess.run([sys.executable, "-m", "pytest"] + args, check=True, cwd=self.model.config[NmkRootConfig.PROJECT_DIR].value)



from pathlib import Path
from typing import List

from nmk.model.resolver import NmkListConfigResolver


class FilesFinder(NmkListConfigResolver):
    def find_in_folders(self) -> List[str]:  # pragma: no cover
        pass

    def get_value(self, name: str) -> List[Path]:
        # Iterate on source paths, and find all python files
        return [src_file for src_path in map(Path, self.find_in_folders()) for src_file in filter(lambda f: f.is_file(), src_path.rglob("*.py"))]


class PythonFilesFinder(FilesFinder):
    def find_in_folders(self) -> List[str]:
        return self.model.config["pythonSrcFolders"].value


class PythonTestFilesFinder(FilesFinder):
    def find_in_folders(self) -> List[str]:
        return [self.model.config["pythonTestSources"].value]



import sys
from typing import List

from nmk.model.builder import NmkTaskBuilder
from nmk.utils import run_with_logs


class FlakeBuilder(NmkTaskBuilder):
    def build(self, src_folders: List[str]):
        # Delegate to flake8
        run_with_logs([sys.executable, "-m", "flake8"] + src_folders, self.logger)

        # Touch output file
        self.main_output.touch()



import shutil
from configparser import ConfigParser
from pathlib import Path
from typing import List, Union

from nmk.model.resolver import NmkListConfigResolver
from nmk_base.common import TemplateBuilder

LIST_SEPARATOR = "\n"


class PythonSupportedVersionsResolver(NmkListConfigResolver):
    def get_value(self, name: str) -> List[str]:
        # Get min/max values, and verify consistency
        min_ver, max_ver = self.model.config["pythonMinVersion"].value, self.model.config["pythonMaxVersion"].value
        min_split, max_split = list(map(int, min_ver.split("."))), list(map(int, max_ver.split(".")))
        prefix = "Inconsistency in python min/max supported versions: "
        assert len(min_split) == len(max_split), prefix + "not the same segments count"
        assert len(min_split) == 2, prefix + "can only deal with X.Y versions (2 segments expected)"
        assert min_split[0] == max_split[0], prefix + "can't deal with different major versions"
        assert max_split[1] > min_split[1], prefix + "max isn't greater than min"

        # Iterate and return versions range
        return [f"{min_split[0]}.{sub}" for sub in range(min_split[1], max_split[1] + 1)]


class PythonSetupBuilder(TemplateBuilder):
    def handle_ini_values(self, values: Union[str, List[str]]):
        # Turn list into a single block of text values
        return (LIST_SEPARATOR + LIST_SEPARATOR.join([self.relative_path(str(v)) for v in values])) if isinstance(values, list) else values

    def build(self, setup_py_template: str, setup_cfg_files: List[str], setup_items: dict):
        # Copy setup.py
        setup_py_output = self.outputs[0]
        shutil.copyfile(Path(setup_py_template), setup_py_output)

        # Merge setup fragments to generate final setup
        setup_cfg_output = self.outputs[1]
        c = ConfigParser()
        for f_path in map(Path, setup_cfg_files):
            # Update config with rendered template
            c.read_string(self.render_template(f_path, {}))

        # Iterate on items contributed through yml project files (only ones contributing maps)
        for section, values in filter(lambda t: isinstance(t[1], dict), setup_items.items()):
            # Create new section is not done yet
            if not c.has_section(section):
                c.add_section(section)

            # Handle list of values
            prepared_values = {k: self.handle_ini_values(v) for k, v in values.items()}

            # Finally update section values
            c[section].update(prepared_values)

        # Finally write config to output file
        with setup_cfg_output.open("w") as f:
            c.write(f)



import sys

from nmk.model.builder import NmkTaskBuilder
from nmk.model.keys import NmkRootConfig
from nmk.model.resolver import NmkStrConfigResolver
from nmk.utils import is_windows, run_with_logs
from nmk_base.venv import VenvUpdateBuilder


class PackageBuilder(NmkTaskBuilder):
    def build(self, setup: str, artifacts: str):
        # Delegate to setup
        run_with_logs(
            [sys.executable, setup, "sdist", "-d", artifacts, "bdist_wheel", "-d", artifacts],
            self.logger,
            cwd=self.model.config[NmkRootConfig.PROJECT_DIR].value,
        )


class PythonPackageForWheel(NmkStrConfigResolver):
    def get_value(self, name: str) -> str:
        return self.model.config["pythonPackage"].value.replace("-", "_")


class Installer(VenvUpdateBuilder):
    def build(self, name: str, pip_args: str):
        # On Windows, refuse to install nmk package while running nmk (wont' work)
        if is_windows() and name == "nmk":
            self.logger.warning("Can't install nmk while running nmk!")
        else:
            super().build(pip_args)



import re

from nmk.model.builder import NmkTaskBuilder
from nmk.model.resolver import NmkStrConfigResolver

# Git version pattern
GIT_VERSION_PATTERN = re.compile("([^-]+)(?:-([0-9]+))?(?:-(.+))?")


class PythonVersionResolver(NmkStrConfigResolver):
    def get_value(self, name: str) -> str:
        # Turn the git version in the Python way
        # See https://www.python.org/dev/peps/pep-0440/
        git_version = self.model.config["gitVersion"].value
        m = GIT_VERSION_PATTERN.match(git_version)
        if m is not None:
            # Build python version from git version segments
            out = m.group(1)
            if m.group(2) is not None:
                # Add commits count
                out += f".post{m.group(2)}"
            if m.group(3) is not None:
                # Add hash/dirty
                out += f"+{m.group(3)}".replace("-", ".")
            return out

        # Probably a simpler version (without segments)
        # Assume it is already compliant...
        return git_version


class PythonVersionRefresh(NmkTaskBuilder):
    def build(self, version: str):
        # Simple version dump
        self.logger.info(self.task.emoji, self.task.description)
        with self.main_output.open("w") as f:
            f.write(version)
